/** observer.dart --- Class for easily notifying registered functions of events.
 *
 * Copyright (C) 2014  Miguel Guedes
 * 
 * Author: Miguel Guedes <miguel.a.guedes@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see http://www.gnu.org/licenses/.
 *
 * Comments:
 *
 * 
 */


import 'observer_events.dart';


/** Manager of notifiable events where clients register a function to receive
 * notifications for a particular event.
 *
 * Instantiate this class if you cannot inherit from Observable. */
class Observer
{
  ObserverEvents                on;
  
  Map<String, List<function> >  _observers;

  
  Observer()
    : _observers = new Map<String, List<function> >()
  {
    on = new ObserverEvents(this);
  }

  bool add(final String   event,
           final function fn)
  {
    if(!_observers.containsKey(event))
      _observers[event] = new List<function>();
    
    if(!_observers[event].contains(fn)) {
      _observers[event].add(fn);
      return true;
    }

    return false;
  }

  bool remove(final String   event,
              final function fn)
  {
    if(_observers.containsKey(event) && _observers[event].contains(fn)) {
      _observers[event].remove(fn);
      return true;
    }

    return false;
  }

  void dispatch(final String event)
  {
    if(_observers.containsKey(event)) {
      for(final function fn in _observers[event])
        fn();
    }
  }
}